#coding=UTF8
from nltk import PunktWordTokenizer
from math import sqrt, pow
from cPickle import dump, load
import sys

# F1-measure = 0.271084337349

#vector space model classifier
#vector is represented by dictionary: key is word, value is a number of the word's occurrences in text
class Solution():
    stopWords=list()
    stemmer=None
    model=None
    tokenizer=None

    # stemmer - class that implements stemming algorithm
    # stopWrds - list of stop words
    def __init__(self, stemmer = None, tokenizer=PunktWordTokenizer().tokenize, stopWords = list()):
        self.stemmer = stemmer
        self.stopWords = stopWords
        self.tokenizer = tokenizer
        try:
            self.loadModel('model.pkl')
        except:
            sys.stderr.write("Warning: model was not loaded!\n")
        
    #process word and add it into counter allWords
    #processing means excluding words in stop list and stemming if stemmer is available
    #Note: stemming in not used in baseline implementation
    def addWord(self, word, counter):
        w = word.lower()
        if w not in self.stopWords:
            if self.stemmer != None:
                ws = self.stemmer.stem(w,0,len(w)-1)
            else:
                ws = w
            counter.setdefault(ws,0)
            counter[ws]+=1
        
    #return vector representation of the document
    #Note: document is treated as a string
    def getVector(self,doc):
        tokenizedDocument = self.tokenizer(doc)
        vectorRepresentation = dict() #this is temporal word counter
        [self.addWord(x,vectorRepresentation) for x in tokenizedDocument] #process words
        return vectorRepresentation
    
    #return cosine similarity between v1 and v2
    def cosineSimilarity(self,v1,v2):
        intersection = set(v1.keys()).intersection(set(v2.keys()))
        if len(intersection)==0: return 0
        def add(x, y): return x+y 
        nominator=reduce(add,[v1[key]*v2[key] for key in intersection])
        denominator=sqrt(reduce(add,[pow(v1[key],2) for key in v1.keys()]))
        denominator*=sqrt(reduce(add,[pow(v2[key],2) for key in v2.keys()]))
        return nominator/denominator
    
    #Training of model
    #listOfExamples consists of pairs [document (as string), class]
    #This function saves vector for each document and its class
    def train(self,listOfExamples):
        self.model=list()
        [self.model.append([self.getVector(example[0]),example[1]]) for example in listOfExamples]
    
    def loadModel(self, path):
        file=open(path)
        self.model=load(file)
        file.close()

    # This function is not used by automatic testing system
    # Here it is presented for example
    def saveModel(self, path):
        file = open(path,"wb")
        dump(self.model,file)
        file.close()
            
    #classify document according to model
    def classify(self,doc):
        vector = self.getVector(doc)
        maxSimilarity = -1
        result = None
        for pair in self.model:
            sim = self.cosineSimilarity(vector,pair[0])
            if sim > maxSimilarity:
                result = pair[1]
                maxSimilarity = sim
        return result

